@@ -1,16 +1,4 @@
 /* alphabet.cc -- File containing alphabets for automata
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include <mata/alphabet.hh>
@@ -61,9 +49,8 @@ Symbol mata::OnTheFlyAlphabet::translate_symb(const std::string& str) {
     const auto it_insert_pair = symbol_map_.insert({str, next_symbol_value_});
     if (it_insert_pair.second) {
         return next_symbol_value_++;
-    } else {
-        return it_insert_pair.first->second;
     }
+    return it_insert_pair.first->second;
 
     // TODO: How can the user specify to throw exceptions when we encounter an unknown symbol? How to specify that
     //  the alphabet should have only the previously fixed symbols?
