@@ -1,18 +1,7 @@
 /*
  * mintermization.hh -- Mintermization of automaton.
+ *
  * It transforms an automaton with a bitvector formula used a symbol to mintermized version of the automaton.
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include "mata/parser/mintermization.hh"
@@ -153,8 +142,8 @@ mata::Mintermization::OptionalBdd mata::Mintermization::graph_to_bdd_afa(const F
         if (symbol_to_bddvar.count(node.name)) {
             return OptionalBdd(symbol_to_bddvar.at(node.name));
         } else {
-            BDD res = (node.name == "true") ? bdd_mng.bddOne() :
-                    (node.name == "false" ? bdd_mng.bddZero() : bdd_mng.bddVar());
+            BDD res = (node.is_true()) ? bdd_mng.bddOne() :
+                    (node.is_false() ? bdd_mng.bddZero() : bdd_mng.bddVar());
             symbol_to_bddvar[node.name] = res;
             return OptionalBdd(res);
         }
@@ -189,8 +178,8 @@ BDD mata::Mintermization::graph_to_bdd_nfa(const FormulaGraph &graph)
         if (symbol_to_bddvar.count(node.name)) {
             return symbol_to_bddvar.at(node.name);
         } else {
-            BDD res = (node.name == "true") ? bdd_mng.bddOne() :
-                      (node.name == "false" ? bdd_mng.bddZero() : bdd_mng.bddVar());
+            BDD res = (node.is_true()) ? bdd_mng.bddOne() :
+                      (node.is_false() ? bdd_mng.bddZero() : bdd_mng.bddVar());
             symbol_to_bddvar[node.name] = res;
             return res;
         }
