@@ -1,8 +1,8 @@
 /* delta.cc -- Operations on Delta.
- *
  */
 // TODO: Add file header.
 
+#include "mata/nfa/types.hh"
 #include "mata/utils/sparse-set.hh"
 #include "mata/nfa/nfa.hh"
 #include "mata/nfa/delta.hh"
@@ -11,6 +11,7 @@
 #include <algorithm>
 #include <list>
 #include <iterator>
+#include <queue>
 
 using namespace mata::utils;
 using namespace mata::nfa;
@@ -39,6 +40,7 @@ void SymbolPost::insert(State s) {
     }
 }
 
+//TODO: slow! This should be doing merge, not inserting one by one.
 void SymbolPost::insert(const StateSet& states) {
     for (State s : states) {
         insert(s);
@@ -59,6 +61,20 @@ StatePost::const_iterator Delta::epsilon_symbol_posts(const StatePost& state_pos
     return state_post.end();
 }
 
+std::vector<Transition> Delta::get_transitions_to(const State state_to) const {
+    std::vector<Transition> transitions_to_state{};
+    const size_t num_of_states{ this->num_of_states() };
+    for (State state_from{ 0 }; state_from < num_of_states; ++state_from) {
+        for (const SymbolPost& state_from_move: state_post(state_from)) {
+            const auto target_state{ state_from_move.targets.find(state_to) };
+            if (target_state != state_from_move.targets.end()) {
+                transitions_to_state.emplace_back(state_from, state_from_move.symbol, state_to);
+            }
+        }
+    }
+    return transitions_to_state;
+}
+
 void Delta::add(State source, Symbol symbol, State target) {
     const State max_state{ std::max(source, target) };
     if (max_state >= state_posts_.size()) {
@@ -137,9 +153,9 @@ void Delta::remove(State src, Symbol symb, State tgt) {
                     "Transition [" + std::to_string(src) + ", " + std::to_string(symb) + ", " +
                     std::to_string(tgt) + "] does not exist.");
         } else {
-            symbol_transitions->remove(tgt);
+            symbol_transitions->erase(tgt);
             if (symbol_transitions->empty()) {
-                state_posts_[src].remove(*symbol_transitions);
+                state_posts_[src].erase(*symbol_transitions);
             }
         }
     }
@@ -174,7 +190,7 @@ size_t Delta::num_of_transitions() const {
     size_t number_of_transitions{ 0 };
     for (const StatePost& state_post: state_posts_) {
         for (const SymbolPost& symbol_post: state_post) {
-            number_of_transitions += symbol_post.size();
+            number_of_transitions += symbol_post.num_of_targets();
         }
     }
     return number_of_transitions;
@@ -187,7 +203,7 @@ bool Delta::empty() const {
     return true;
 }
 
-Delta::Transitions::const_iterator::const_iterator(const Delta& delta): delta_{ &delta }, current_state_{ 0 } {
+Delta::Transitions::const_iterator::const_iterator(const Delta& delta): delta_{ &delta } {
     const size_t post_size = delta_->num_of_states();
     for (size_t i = 0; i < post_size; ++i) {
         if (!(*delta_)[i].empty()) {
@@ -287,11 +303,11 @@ std::vector<StatePost> Delta::renumber_targets(const std::function<State(State)>
         copied_state_post.reserve(state_post.size());
         for(const SymbolPost& symbol_post: state_post) {
             StateSet copied_targets;
-            copied_targets.reserve(symbol_post.size());
+            copied_targets.reserve(symbol_post.num_of_targets());
             for(const State& state: symbol_post.targets) {
                 copied_targets.push_back(std::move(target_renumberer(state)));
             }
-            copied_state_post.push_back(std::move(SymbolPost(symbol_post.symbol, copied_targets)));
+            copied_state_post.push_back(SymbolPost(symbol_post.symbol, copied_targets));
         }
         copied_state_posts.emplace_back(copied_state_post);
     }
@@ -360,6 +376,24 @@ bool Delta::operator==(const Delta& other) const {
     return other_transitions_it == other_transitions_end;
 }
 
+///Returns an iterator to the smallest epsilon, or end() if there is no epsilon
+///Searches from the end of the vector of SymbolPosts, since epsilons are at the end and they are typically few, mostly 1.
+StatePost::const_iterator StatePost::first_epsilon_it(Symbol first_epsilon) const {
+    auto end_it = end();
+    auto it = end_it;
+    while (it != begin()) {
+        --it;
+        if (it->symbol < first_epsilon) { //is it a normal symbol already?
+            return it + 1; // Return the previous position, the smallest epsilon or end().
+        }
+    }
+
+    if (it != end_it && it->symbol >= first_epsilon) //special case when begin is the smallest epsilon (since the while loop ended before the step back)
+        return it;
+
+    return end_it;
+}
+
 StatePost::Moves::const_iterator::const_iterator(
     const StatePost& state_post, const StatePost::const_iterator symbol_post_it,
     const StatePost::const_iterator symbol_post_end)
@@ -426,7 +460,7 @@ bool StatePost::Moves::const_iterator::operator==(const StatePost::Moves::const_
 size_t StatePost::num_of_moves() const {
     size_t counter{ 0 };
     for (const SymbolPost& symbol_post: *this) {
-        counter += symbol_post.size();
+        counter += symbol_post.num_of_targets();
     }
     return counter;
 }
@@ -455,47 +489,12 @@ StatePost::Moves StatePost::moves(
 }
 
 StatePost::Moves StatePost::moves_epsilons(const Symbol first_epsilon) const {
-    const StatePost::const_iterator symbol_post_begin{ cbegin() };
-    const StatePost::const_iterator symbol_post_end{ cend() };
-    if (empty()) {
-        return { *this, symbol_post_end, symbol_post_end };
-    }
-    if (symbol_post_begin->symbol >= first_epsilon) {
-        return { *this, symbol_post_begin, symbol_post_end };
-    }
-
-    StatePost::const_iterator previous_symbol_post_it{ std::prev(symbol_post_end) };
-    StatePost::const_iterator symbol_post_it{ previous_symbol_post_it };
-    while (previous_symbol_post_it != symbol_post_begin && first_epsilon < previous_symbol_post_it->symbol) {
-        symbol_post_it = previous_symbol_post_it;
-        --previous_symbol_post_it;
-    }
-    if (first_epsilon <= previous_symbol_post_it->symbol) {
-        return { *this, previous_symbol_post_it, symbol_post_end };
-    }
-    if (first_epsilon <= symbol_post_it->symbol) {
-        return { *this, symbol_post_it, symbol_post_end };
-    }
-    return { *this, symbol_post_end, symbol_post_end };
+    return { *this, first_epsilon_it(first_epsilon), cend() };
 }
 
 StatePost::Moves StatePost::moves_symbols(const Symbol last_symbol) const {
-    const StatePost::const_iterator symbol_post_end{ cend() };
-    const StatePost::const_iterator symbol_post_begin{ cbegin() };
-    if (empty() || symbol_post_begin->symbol > last_symbol) {
-        return { *this, symbol_post_end, symbol_post_end };
-    }
-
-    StatePost::const_iterator end_symbol_post_it { symbol_post_end };
-    StatePost::const_iterator previous_end_symbol_post_it{ std::prev(symbol_post_end) };
-    while (previous_end_symbol_post_it != symbol_post_begin && last_symbol < previous_end_symbol_post_it->symbol) {
-        end_symbol_post_it = previous_end_symbol_post_it;
-        --previous_end_symbol_post_it;
-    }
-    // Either previous_end_symbol_post_it is == symbol_post_begin, at which case we should iterate only over the first
-    //  symbol post (that is, end_symbol_post_it == symbol_post_begin + 1); or, previous_end_symbol_post_it jumped over
-    //  last_symbol and end_symbol_post_it is the first symbol post (or end()) after last symbol.
-    return { *this, symbol_post_begin, end_symbol_post_it };
+    if (last_symbol == EPSILON) { throw std::runtime_error("Using default epsilon as a last symbol to iterate over."); }
+    return { *this, cbegin(), first_epsilon_it(last_symbol + 1) };
 }
 
 StatePost::Moves::const_iterator StatePost::Moves::begin() const {
@@ -512,3 +511,229 @@ Delta::Transitions::const_iterator Delta::Transitions::end() const { return cons
 StatePost::Moves::Moves(
     const StatePost& state_post, StatePost::const_iterator symbol_post_it, StatePost::const_iterator symbol_post_end)
     : state_post_{ &state_post }, symbol_post_it_{ symbol_post_it }, symbol_post_end_{ symbol_post_end } {}
+
+void Delta::add_symbols_to(OnTheFlyAlphabet& target_alphabet) const {
+    size_t aut_num_of_states{ num_of_states() };
+    for (mata::nfa::State state{ 0 }; state < aut_num_of_states; ++state) {
+        for (const SymbolPost& move: state_post(state)) {
+            target_alphabet.update_next_symbol_value(move.symbol);
+            target_alphabet.try_add_new_symbol(std::to_string(move.symbol), move.symbol);
+        }
+    }
+}
+
+OrdVector<Symbol> Delta::get_used_symbols() const {
+    //TODO: look at the variants in profiling (there are tests in tests-nfa-profiling.cc),
+    // for instance figure out why NumberPredicate and OrdVector are slow,
+    // try also with _STATIC_DATA_STRUCTURES_, it changes things.
+
+    //below are different variant, with different data structures for accumulating symbols,
+    //that then must be converted to an OrdVector
+    //measured are times with "mata::nfa::get_used_symbols speed, harder", "[.profiling]" now on line 104 of nfa-profiling.cc
+
+    //WITH VECTOR (4.434 s)
+    return get_used_symbols_vec();
+
+    //WITH SET (26.5 s)
+    //auto from_set = get_used_symbols_set();
+    //return utils::OrdVector<Symbol> (from_set .begin(),from_set.end());
+
+    //WITH NUMBER PREDICATE (4.857s) (NP removed)
+    //return utils::OrdVector(get_used_symbols_np().get_elements());
+
+    //WITH SPARSE SET (haven't tried)
+    //return utils::OrdVector<State>(get_used_symbols_sps());
+
+    //WITH BOOL VECTOR (error !!!!!!!):
+    //return utils::OrdVector<Symbol>(utils::NumberPredicate<Symbol>(get_used_symbols_bv()));
+
+    //WITH BOOL VECTOR (1.9s): (The fastest, it seems.)
+    // However, it will try to allocate a vector indexed by the symbols. If there are epsilons in the automaton,
+    //  for example, the bool vector implementation will implode.
+    // std::vector<bool> bv{ get_used_symbols_bv() };
+    // utils::OrdVector<Symbol> ov{};
+    // const size_t bv_size{ bv.size() };
+    // for (Symbol i{ 0 }; i < bv_size; ++i) { if (bv[i]) { ov.push_back(i); } }
+    // return ov;
+
+    ///WITH BOOL VECTOR, DIFFERENT VARIANT? (1.9s):
+    //std::vector<bool> bv = get_used_symbols_bv();
+    //std::vector<Symbol> v(std::count(bv.begin(), bv.end(), true));
+    //return utils::OrdVector<Symbol>(v);
+
+    //WITH CHAR VECTOR (should be the fastest, haven't tried in this branch):
+    //BEWARE: failing in one noodlificatoin test ("Simple automata -- epsilon result") ... strange
+    //BoolVector chv = get_used_symbols_chv();
+    //utils::OrdVector<Symbol> ov;
+    //for(Symbol i = 0;i<chv.size();i++)
+    //    if (chv[i]) {
+    //        ov.push_back(i);
+    //    }
+    //return ov;
+}
+
+// Other versions, maybe an interesting experiment with speed of data structures.
+// Returns symbols appearing in Delta, pushes back to vector and then sorts
+mata::utils::OrdVector<Symbol> Delta::get_used_symbols_vec() const {
+#ifdef _STATIC_STRUCTURES_
+    static std::vector<Symbol> symbols{};
+    symbols.clear();
+#else
+    std::vector<Symbol> symbols{};
+#endif
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost & symbol_post: state_post) {
+            utils::reserve_on_insert(symbols);
+            symbols.push_back(symbol_post.symbol);
+        }
+    }
+    utils::OrdVector<Symbol> sorted_symbols(symbols);
+    return sorted_symbols;
+}
+
+// returns symbols appearing in Delta, inserts to a std::set
+std::set<Symbol> Delta::get_used_symbols_set() const {
+    //static should prevent reallocation, seems to speed things up a little
+#ifdef _STATIC_STRUCTURES_
+    static std::set<Symbol>  symbols;
+    symbols.clear();
+#else
+    static std::set<Symbol>  symbols{};
+#endif
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost& symbol_post: state_post) {
+            symbols.insert(symbol_post.symbol);
+        }
+    }
+    return symbols;
+    //utils::OrdVector<Symbol>  sorted_symbols(symbols.begin(),symbols.end());
+    //return sorted_symbols;
+}
+
+// returns symbols appearing in Delta, adds to NumberPredicate,
+// Seems to be the fastest option, but could have problems with large maximum symbols
+mata::utils::SparseSet<Symbol> Delta::get_used_symbols_sps() const {
+#ifdef _STATIC_STRUCTURES_
+    //static seems to speed things up a little
+    static utils::SparseSet<Symbol> symbols(64,false);
+    symbols.clear();
+#else
+    utils::SparseSet<Symbol> symbols(64);
+#endif
+    //symbols.dont_track_elements();
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost & symbol_post: state_post) {
+            symbols.insert(symbol_post.symbol);
+        }
+    }
+    //TODO: is it necessary to return ordered vector? Would the number predicate suffice?
+    return symbols;
+}
+
+// returns symbols appearing in Delta, adds to NumberPredicate,
+// Seems to be the fastest option, but could have problems with large maximum symbols
+std::vector<bool> Delta::get_used_symbols_bv() const {
+#ifdef _STATIC_STRUCTURES_
+    //static seems to speed things up a little
+    static std::vector<bool> symbols(64, false);
+    symbols.clear();
+#else
+    std::vector<bool> symbols(64, false);
+#endif
+    //symbols.dont_track_elements();
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost& symbol_post: state_post) {
+            const size_t capacity{ symbol_post.symbol + 1 };
+            reserve_on_insert(symbols, capacity);
+            if (symbols.size() < capacity) {
+                symbols.resize(capacity);
+            }
+            symbols[symbol_post.symbol] = true;
+        }
+    }
+    return symbols;
+}
+
+mata::BoolVector Delta::get_used_symbols_chv() const {
+#ifdef _STATIC_STRUCTURES_
+    //static seems to speed things up a little
+    static BoolVector symbols(64,false);
+    symbols.clear();
+#else
+    BoolVector symbols(64,false);
+#endif
+    //symbols.dont_track_elements();
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost& symbol_post: state_post) {
+            reserve_on_insert(symbols,symbol_post.symbol);
+            symbols[symbol_post.symbol] = true;
+        }
+    }
+    //TODO: is it necessary to return ordered vector? Would the number predicate suffice?
+    return symbols;
+}
+
+Symbol Delta::get_max_symbol() const {
+    Symbol max{ 0 };
+    for (const StatePost& state_post: state_posts_) {
+        for (const SymbolPost& symbol_post: state_post) {
+            if (symbol_post.symbol > max) { max = symbol_post.symbol; }
+        }
+    }
+    return max;
+}
+
+StateSet SynchronizedExistentialSymbolPostIterator::unify_targets() const {
+    // TODO: decide which version performs the best.
+
+    if(!is_synchronized()) { return {}; }
+
+    StateSet unified_targets{};
+
+    // Version with synchronized iterator.
+    // static utils::SynchronizedExistentialIterator<StateSet::const_iterator> sync_iterator;
+    // sync_iterator.reset();
+    // size_t all_targets_size{ 0 };
+    // const std::vector<StatePost::const_iterator>& current_symbol_post_its{ this->get_current() };
+    // sync_iterator.reserve(current_symbol_post_its.size());
+    // for (const auto symbol_post_it: current_symbol_post_its) {
+    //     sync_iterator.push_back(symbol_post_it->cbegin(), symbol_post_it->cend());
+    //     all_targets_size += symbol_post_it->num_of_targets();
+    // }
+    // unified_targets.reserve(all_targets_size);
+    // while (sync_iterator.advance()) { unified_targets.push_back(*sync_iterator.get_current_minimum()); }
+
+    // Version with set union.
+    // for (const auto& symbol_post_it: get_current()) {
+    //     unified_targets.insert(symbol_post_it->targets);
+    // }
+
+    // Version with priority queue.
+    using TargetSetBeginEndPair = std::pair<StateSet::const_iterator, StateSet::const_iterator>;
+    auto compare = [](const auto& a, const auto& b) { return *(a.first) > *(b.first); };
+    std::priority_queue<TargetSetBeginEndPair, std::vector<TargetSetBeginEndPair>, decltype(compare) > queue(compare);
+    for (const StatePost::const_iterator& symbol_post_it: get_current()) {
+        queue.emplace(symbol_post_it->cbegin(), symbol_post_it->cend());
+    }
+    unified_targets.reserve(32);
+    while (!queue.empty()) {
+        auto item = queue.top();
+        queue.pop();
+        if (unified_targets.empty() || unified_targets.back() != *(item.first)) {
+            unified_targets.push_back(*(item.first));
+        }
+        if (++item.first != item.second) { queue.emplace(item); }
+    }
+
+    return unified_targets;
+}
+
+bool SynchronizedExistentialSymbolPostIterator::synchronize_with(const SymbolPost& sync) {
+    do {
+        if (is_synchronized()) {
+            auto current_min_symbol_post_it = get_current_minimum();
+            if (*current_min_symbol_post_it >= sync) { break; }
+        }
+    } while (advance());
+    return is_synchronized() && *get_current_minimum() == sync;
+}
