@@ -1,24 +1,9 @@
 /* nfa.cc -- operations for NFA
- *
- * Copyright (c) 2018 Ondrej Lengal <ondra.lengal@gmail.com>
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include <algorithm>
 #include <list>
 #include <optional>
-#include <unordered_set>
 #include <iterator>
 
 // MATA headers
@@ -54,7 +39,7 @@ namespace {
     StateBoolArray reachable_states(const Nfa& nfa,
                                        const std::optional<const StateBoolArray>& states_to_consider = std::nullopt) {
         std::vector<State> worklist{};
-        StateBoolArray reachable(nfa.size(), false);
+        StateBoolArray reachable(nfa.num_of_states(), false);
         for (const State state: nfa.initial) {
             if (!states_to_consider.has_value() || states_to_consider.value()[state]) {
                 worklist.push_back(state);
@@ -89,7 +74,7 @@ StateSet Nfa::get_reachable_states() const {
     StateBoolArray reachable_bool_array{ reachable_states(*this) };
 
     StateSet reachable_states{};
-    const size_t num_of_states{size() };
+    const size_t num_of_states{ this->num_of_states() };
     for (State original_state{ 0 }; original_state < num_of_states; ++original_state)
     {
         if (reachable_bool_array[original_state])
@@ -106,6 +91,32 @@ StateSet Nfa::get_terminating_states() const
     return revert(*this).get_reachable_states();
 }
 
+std::vector<State> Nfa::distances_from_initial() const {
+    std::vector<State> distances(num_of_states()+1, Limits::max_state);
+    BoolVector visited(num_of_states()+1, false);
+    std::deque<State> que;
+
+    for (State qi: initial) {
+        visited[qi] = true;
+        distances[qi] = 0;
+        que.push_back(qi);
+    }
+
+    while (!que.empty()) {
+        State src = que.front();
+        que.pop_front();
+        for (Move move : delta[src].moves()) {
+            if (!visited[move.target]) {
+                visited[move.target] = true;
+                distances[move.target] = distances[src] +1;
+                que.push_back(move.target);
+            }
+        }
+    }
+
+    return distances;
+}
+
 Nfa& Nfa::trim(StateRenaming* state_renaming) {
 #ifdef _STATIC_STRUCTURES_
     BoolVector useful_states{ useful_states() };
@@ -150,10 +161,8 @@ namespace {
     // of useful states. It contains Tarjan's metadata and the state of the
     // iteration through the successors.
     struct TarjanNodeData {
-        StatePost::const_iterator post_it{};
-        StatePost::const_iterator post_end{};
-        StateSet::const_iterator targets_it{};
-        StateSet::const_iterator targets_end{};
+        StatePost::Moves::const_iterator current_move{};
+        StatePost::Moves::const_iterator end_move{};
         // index of a node (corresponds to the time of discovery)
         unsigned long index{ 0 };
         // index of a lower node in the same SCC
@@ -165,46 +174,12 @@ namespace {
 
         TarjanNodeData() = default;
 
-        TarjanNodeData(State q, const Delta & delta, unsigned long index)
-            : post_it(delta[q].cbegin()), post_end(delta[q].cend()), index(index), lowlink(index), initilized(true), on_stack(true) {
-            if (post_it != post_end) {
-                targets_it = post_it->cbegin();
-                targets_end = post_it->cend();
-            }
+        TarjanNodeData(State q, const Delta& delta, unsigned long index)
+            : index(index), lowlink(index), initilized(true), on_stack(true) {
+            const StatePost::Moves moves{ delta[q].moves() };
+            current_move = moves.begin();
+            end_move = moves.end();
         };
-
-        // TODO: this sucks. In fact, if you want to check that you have the last sucessor, you need to
-        // first align the iterators.
-        // TODO: this is super-ugly. If we introduce Post::transitions iterator, this could be much easier.
-        // Align iterators in a way that the current state is stored in *(this->targets_it).
-        void align_succ() {
-            while (this->post_it != this->post_end && this->targets_it == this->targets_end) {
-                if (this->targets_it == this->targets_end) {
-                    ++this->post_it;
-                    if (this->post_it != this->post_end) {
-                        this->targets_it = this->post_it->cbegin();
-                        this->targets_end = this->post_it->cend();
-                    }
-                }
-            }
-        }
-
-        State get_curr_succ() {
-            align_succ();
-            return *(this->targets_it);
-        }
-
-        void move_to_next_succ() {
-            if(this->post_it == this->post_end) {
-                return;
-            }
-            ++this->targets_it;
-        }
-
-        bool is_end_succ() {
-            align_succ();
-            return this->post_it == this->post_end;
-        }
     };
 };
 
@@ -242,13 +217,11 @@ namespace {
  *    SCC if useful, we declare all nodes in the SCC useful and moreover we propagate usefulness also the states
  *    in @p tarjan_stack as it contains states that can reach this closed SCC.
  *
- * @return BoolVector
  */
-BoolVector Nfa::get_useful_states() const {
-    BoolVector useful(this->size(),false);
-    std::vector<TarjanNodeData> node_info(this->size());
-    std::deque<State> program_stack;
-    std::deque<State> tarjan_stack;
+void Nfa::tarjan_scc_discover(const TarjanDiscoverCallback& callback) const {
+    std::vector<TarjanNodeData> node_info(this->num_of_states());
+    std::vector<State> program_stack;
+    std::vector<State> tarjan_stack;
     unsigned long index_cnt = 0;
 
     for(const State& q0 : initial) {
@@ -271,14 +244,15 @@ BoolVector Nfa::get_useful_states() const {
             // initialize node
             act_state_data = TarjanNodeData(act_state, this->delta, index_cnt++);
             tarjan_stack.push_back(act_state);
-            if(this->final.contains(act_state)) {
-                useful[act_state] = true;
+
+            if(callback.state_discover && callback.state_discover(act_state)) {
+                return;
             }
         } else { // return from the recursive call
-            State act_succ = act_state_data.get_curr_succ();
+            State act_succ = act_state_data.current_move->target;
             act_state_data.lowlink = std::min(act_state_data.lowlink, node_info[act_succ].lowlink);
             // act_succ is the state that cased the recursive call. Move to another successor.
-            act_state_data.move_to_next_succ();
+            ++act_state_data.current_move;
         }
 
         // iterate through outgoing edges
@@ -286,11 +260,10 @@ BoolVector Nfa::get_useful_states() const {
         // rec_call simulates call of the strongconnect. Since c++ cannot do continue over
         // multiple loops, we use rec_call to jump to the main loop
         bool rec_call = false;
-        while(!act_state_data.is_end_succ()) {
-            next_state = act_state_data.get_curr_succ();
-            // if successor is useful, act_state is useful as well
-            if(useful[next_state]) {
-                useful[act_state] = true;
+        for(; act_state_data.current_move != act_state_data.end_move; ++act_state_data.current_move) {
+            next_state = act_state_data.current_move->target;
+            if(callback.succ_state_discover) {
+                callback.succ_state_discover(act_state, next_state);
             }
             if(!node_info[next_state].initilized) { // recursive call
                 program_stack.push_back(next_state);
@@ -299,47 +272,148 @@ BoolVector Nfa::get_useful_states() const {
             } else if(node_info[next_state].on_stack) {
                 act_state_data.lowlink = std::min(act_state_data.lowlink, node_info[next_state].index);
             }
-            act_state_data.move_to_next_succ();
         }
         if(rec_call) continue;
 
         // check if we have the root of a SCC
         if(act_state_data.lowlink == act_state_data.index) {
             State st;
-            // contains the closed SCC a final state
-            bool final_scc = false;
             std::vector<State> scc;
             do {
                 st = tarjan_stack.back();
                 tarjan_stack.pop_back();
                 node_info[st].on_stack = false;
 
-                // SCC contains a final state
-                if(useful[st]) {
-                    final_scc = true;
+                if(callback.scc_state_discover) {
+                    callback.scc_state_discover(st);
                 }
                 scc.push_back(st);
             } while(st != act_state);
-            if(final_scc) {
-                // propagate usefulness to the closed SCC
-                for(const State& st : scc) useful[st] = true;
-                // propagate usefulness to predecessors in @p tarjan_stack
-                for(const State& st : tarjan_stack) useful[st] = true;
+            if(callback.scc_discover && callback.scc_discover(scc, tarjan_stack)) {
+                return;
             }
         }
         // all successors have been processed, we can remove act_state from the program stack
         program_stack.pop_back();
     }
+}
+
+BoolVector Nfa::get_useful_states() const {
+    BoolVector useful(this->num_of_states(), false);
+    bool final_scc = false;
+
+    TarjanDiscoverCallback callback {};
+    callback.state_discover = [&](State state) -> bool {
+        if(this->final.contains(state)) {
+            useful[state] = true;
+        }
+        return false;
+    };
+    callback.scc_discover = [&](const std::vector<State>& scc, const std::vector<State>& tarjan_stack) -> bool {
+        if(final_scc) {
+            // Propagate usefulness to the closed SCC.
+            for(const State& st: scc) { useful[st] = true; }
+            // Propagate usefulness to predecessors in @p tarjan_stack.
+            for (auto state_it{ tarjan_stack.rbegin() }, state_it_end{ tarjan_stack.rend() };
+                    state_it != state_it_end; ++state_it) {
+                if (useful[*state_it]) { break; }
+                useful[*state_it] = true;
+            }
+        }
+        final_scc = false;
+        return false;
+    };
+    callback.scc_state_discover = [&](State state) {
+        if(useful[state]) {
+            final_scc = true;
+        }
+    };
+    callback.succ_state_discover = [&](State act_state, State next_state) {
+        if(useful[next_state]) {
+            useful[act_state] = true;
+        }
+    };
+
+    tarjan_scc_discover(callback);
     return useful;
 }
 
-std::string Nfa::print_to_DOT() const {
+bool Nfa::is_lang_empty_scc() const {
+    bool accepting_state = false;
+
+    TarjanDiscoverCallback callback {};
+    callback.state_discover = [&](State state) -> bool {
+        if(this->final.contains(state)) {
+            accepting_state = true;
+            return true;
+        }
+        return false;
+    };
+
+    tarjan_scc_discover(callback);
+    return !accepting_state;
+}
+
+bool Nfa::is_acyclic() const {
+    bool acyclic = true;
+
+    TarjanDiscoverCallback callback {};
+    callback.scc_discover = [&](const std::vector<State>& scc, const std::vector<State>& tarjan_stack) -> bool {
+        (void)tarjan_stack;
+        if(scc.size() > 1) {
+            acyclic = false;
+                return true;
+            } else { // check for self-loops
+                State st = scc[0];
+                for(const auto& sp : this->delta[st]) {
+                    if(sp.targets.find(st) != sp.targets.end()) {
+                        acyclic = false;
+                        return true;
+                    }
+                }
+            }
+            return false;
+    };
+
+    tarjan_scc_discover(callback);
+    return acyclic;
+}
+
+bool Nfa::is_flat() const {
+    bool flat = true;
+
+    mata::nfa::Nfa::TarjanDiscoverCallback callback {};
+    callback.scc_discover = [&](const std::vector<mata::nfa::State>& scc, const std::vector<mata::nfa::State>& tarjan_stack) -> bool {
+        (void)tarjan_stack;
+        
+        for(const mata::nfa::State& st : scc) {
+            bool one_input_visited = false;
+            for (const mata::nfa::SymbolPost& sp : this->delta[st]) {
+                for (const mata::nfa::State& tgt : scc) {
+                    if(sp.targets.contains(tgt)) {
+                        if(one_input_visited) {
+                            flat = false;
+                            return true;
+                        }
+                        one_input_visited = true;
+                    }
+                }
+            }
+        }
+        return false;
+    };
+
+    tarjan_scc_discover(callback);
+    return flat;
+}
+
+std::string Nfa::print_to_dot() const {
     std::stringstream output;
-    print_to_DOT(output);
+    print_to_dot(output);
     return output.str();
 }
 
-void Nfa::print_to_DOT(std::ostream &output) const {
+void Nfa::print_to_dot(std::ostream &output) const {
     output << "digraph finiteAutomaton {" << std::endl
                  << "node [shape=circle];" << std::endl;
 
@@ -400,7 +474,7 @@ void Nfa::print_to_mata(std::ostream &output) const {
 }
 
 Nfa Nfa::get_one_letter_aut(Symbol abstract_symbol) const {
-    Nfa digraph{size(), StateSet(initial), StateSet(final) };
+    Nfa digraph{num_of_states(), StateSet(initial), StateSet(final) };
     // Add directed transitions for digraph.
     for (const Transition& transition: delta.transitions()) {
         // Directly try to add the transition. Finding out whether the transition is already in the digraph
@@ -414,20 +488,6 @@ void Nfa::get_one_letter_aut(Nfa& result) const {
     result = get_one_letter_aut();
 }
 
-std::vector<Transition> Nfa::get_transitions_to(State state_to) const {
-    std::vector<Transition> transitions_to_state{};
-    const size_t num_of_states{ delta.num_of_states() };
-    for (State state_from{ 0 }; state_from < num_of_states; ++state_from) {
-        for (const SymbolPost& state_from_move: delta[state_from]) {
-            const auto target_state{ state_from_move.targets.find(state_to) };
-            if (target_state != state_from_move.targets.end()) {
-                transitions_to_state.emplace_back(state_from, state_from_move.symbol, state_to );
-            }
-        }
-    }
-    return transitions_to_state;
-}
-
 StateSet Nfa::post(const StateSet& states, const Symbol& symbol) const {
     StateSet res{};
     if (delta.empty()) {
@@ -436,6 +496,7 @@ StateSet Nfa::post(const StateSet& states, const Symbol& symbol) const {
 
     for (const State state: states) {
         const StatePost& post{ delta[state] };
+        // TODO: This does not handle epsilons.
         const auto move_it{ post.find(symbol) };
         if (move_it != post.end()) {
             res.insert(move_it->targets);
@@ -444,122 +505,6 @@ StateSet Nfa::post(const StateSet& states, const Symbol& symbol) const {
     return res;
 }
 
-// Other versions, maybe an interesting experiment with speed of data structures.
-// Returns symbols appearing in Delta, pushes back to vector and then sorts
-mata::utils::OrdVector<Symbol> Nfa::get_used_symbols_vec() const {
-#ifdef _STATIC_STRUCTURES_
-    static std::vector<Symbol> symbols{};
-    symbols.clear();
-#else
-    std::vector<Symbol>  symbols{};
-#endif
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            utils::reserve_on_insert(symbols);
-            symbols.push_back(move.symbol);
-        }
-    }
-    utils::OrdVector<Symbol>  sorted_symbols(symbols);
-    return sorted_symbols;
-}
-
-// returns symbols appearing in Delta, inserts to a std::set
-std::set<Symbol> Nfa::get_used_symbols_set() const {
-    //static should prevent reallocation, seems to speed things up a little
-#ifdef _STATIC_STRUCTURES_
-    static std::set<Symbol>  symbols;
-    symbols.clear();
-#else
-    static std::set<Symbol>  symbols{};
-#endif
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            symbols.insert(move.symbol);
-        }
-    }
-    return symbols;
-    //utils::OrdVector<Symbol>  sorted_symbols(symbols.begin(),symbols.end());
-    //return sorted_symbols;
-}
-
-// returns symbols appearing in Delta, adds to NumberPredicate,
-// Seems to be the fastest option, but could have problems with large maximum symbols
-mata::utils::SparseSet<Symbol> Nfa::get_used_symbols_sps() const {
-#ifdef _STATIC_STRUCTURES_
-    //static seems to speed things up a little
-    static utils::SparseSet<Symbol>  symbols(64,false);
-    symbols.clear();
-#else
-    utils::SparseSet<Symbol>  symbols(64);
-#endif
-    //symbols.dont_track_elements();
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            symbols.insert(move.symbol);
-        }
-    }
-    //TODO: is it necessary to return ordered vector? Would the number predicate suffice?
-    return symbols;
-}
-
-// returns symbols appearing in Delta, adds to NumberPredicate,
-// Seems to be the fastest option, but could have problems with large maximum symbols
-std::vector<bool> Nfa::get_used_symbols_bv() const {
-#ifdef _STATIC_STRUCTURES_
-    //static seems to speed things up a little
-    static std::vector<bool>  symbols(64,false);
-    symbols.clear();
-#else
-    std::vector<bool> symbols(64,false);
-#endif
-    //symbols.dont_track_elements();
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            reserve_on_insert(symbols,move.symbol);
-            symbols[move.symbol]=true;
-        }
-    }
-    //TODO: is it neccessary toreturn ordered vector? Would the number predicate suffice?
-    return symbols;
-}
-
-BoolVector Nfa::get_used_symbols_chv() const {
-#ifdef _STATIC_STRUCTURES_
-    //static seems to speed things up a little
-    static BoolVector  symbols(64,false);
-    symbols.clear();
-#else
-    BoolVector symbols(64,false);
-#endif
-    //symbols.dont_track_elements();
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            reserve_on_insert(symbols,move.symbol);
-            symbols[move.symbol]=true;
-        }
-    }
-    //TODO: is it necessary to return ordered vector? Would the number predicate suffice?
-    return symbols;
-}
-
-// returns max non-e symbol in Delta
-Symbol Nfa::get_max_symbol() const {
-    Symbol max = 0;
-    for (State q = 0; q< delta.num_of_states(); ++q) {
-        const StatePost & post = delta[q];
-        for (const SymbolPost & move: post) {
-            if (move.symbol > max)
-                max = move.symbol;
-        }
-    }
-    return max;
-}
-
  void Nfa::unify_initial() {
     if (initial.empty() || initial.size() == 1) { return; }
     const State new_initial_state{add_state() };
@@ -580,7 +525,7 @@ void Nfa::unify_final() {
     if (final.empty() || final.size() == 1) { return; }
     const State new_final_state{ add_state() };
     for (const auto& orig_final_state: final) {
-        const auto transitions_to{ get_transitions_to(orig_final_state) };
+        const auto transitions_to{ delta.get_transitions_to(orig_final_state) };
         for (const auto& transitions: transitions_to) {
             delta.add(transitions.source, transitions.symbol, new_final_state);
         }
@@ -590,16 +535,6 @@ void Nfa::unify_final() {
     final.insert(new_final_state);
 }
 
-void Nfa::add_symbols_to(OnTheFlyAlphabet& target_alphabet) const {
-    size_t aut_num_of_states{size() };
-    for (mata::nfa::State state{ 0 }; state < aut_num_of_states; ++state) {
-        for (const SymbolPost& move: delta[state]) {
-            target_alphabet.update_next_symbol_value(move.symbol);
-            target_alphabet.try_add_new_symbol(std::to_string(move.symbol), move.symbol);
-        }
-    }
-}
-
 Nfa& Nfa::operator=(Nfa&& other) noexcept {
     if (this != &other) {
         delta = std::move(other.delta);
@@ -612,31 +547,24 @@ Nfa& Nfa::operator=(Nfa&& other) noexcept {
     return *this;
 }
 
-void Nfa::clear_transitions() {
-    const size_t delta_size = delta.num_of_states();
-    for (size_t i = 0; i < delta_size; ++i) {
-        delta.mutable_state_post(i) = StatePost();
-    }
-}
-
 State Nfa::add_state() {
-    const size_t num_of_states{ size() };
-    delta.increase_size(num_of_states + 1);
+    const size_t num_of_states{ this->num_of_states() };
+    delta.allocate(num_of_states + 1);
     return num_of_states;
 }
 
 State Nfa::add_state(State state) {
     if (state >= delta.num_of_states()) {
-        delta.increase_size(state + 1);
+        delta.allocate(state + 1);
     }
     return state;
 }
 
-size_t Nfa::size() const {
+size_t Nfa::num_of_states() const {
     return std::max({
-        static_cast<unsigned long>(initial.domain_size()),
-        static_cast<unsigned long>(final.domain_size()),
-        static_cast<unsigned long>(delta.num_of_states())
+        static_cast<size_t>(initial.domain_size()),
+        static_cast<size_t>(final.domain_size()),
+        static_cast<size_t>(delta.num_of_states())
     });
 }
 
@@ -646,7 +574,7 @@ void Nfa::clear() {
     final.clear();
 }
 
-bool Nfa::is_identical(const Nfa& aut) {
+bool Nfa::is_identical(const Nfa& aut) const {
     if (utils::OrdVector<State>(initial) != utils::OrdVector<State>(aut.initial)) {
         return false;
     }
@@ -656,52 +584,48 @@ bool Nfa::is_identical(const Nfa& aut) {
     return delta == aut.delta;
 }
 
-OrdVector<Symbol> Nfa::get_used_symbols() const {
-    //TODO: look at the variants in profiling (there are tests in tests-nfa-profiling.cc),
-    // for instance figure out why NumberPredicate and OrdVedctor are slow,
-    // try also with _STATIC_DATA_STRUCTURES_, it changes things.
-
-    //below are different variant, with different data structures for accumulating symbols,
-    //that then must be converted to an OrdVector
-    //measured are times with "mata::nfa::get_used_symbols speed, harder", "[.profiling]" now on line 104 of nfa-profiling.cc
-
-    //WITH VECTOR (4.434 s)
-    //return get_used_symbols_vec();
-
-    //WITH SET (26.5 s)
-    //auto from_set = get_used_symbols_set();
-    //return utils::OrdVector<Symbol> (from_set .begin(),from_set.end());
-
-    //WITH NUMBER PREDICATE (4.857s) (NP removed)
-    //return utils::OrdVector(get_used_symbols_np().get_elements());
-
-    //WITH SPARSE SET (haven't tried)
-    //return utils::OrdVector<State>(get_used_symbols_sps());
-
-    //WITH BOOL VECTOR (error !!!!!!!):
-    //return utils::OrdVector<Symbol>(utils::NumberPredicate<Symbol>(get_used_symbols_bv()));
-
-    //WITH BOOL VECTOR (1.9s):
-    std::vector<bool> bv = get_used_symbols_bv();
-    utils::OrdVector<Symbol> ov;
-    for(Symbol i = 0;i<bv.size();i++)
-        if (bv[i]) {
-            ov.push_back(i);
-        }
-    return ov;
-
-    ///WITH BOOL VECTOR, DIFFERENT VARIANT? (1.9s):
-    //std::vector<bool> bv = get_used_symbols_bv();
-    //std::vector<Symbol> v(std::count(bv.begin(), bv.end(), true));
-    //return utils::OrdVector<Symbol>(v);
-
-    //WITH CHAR VECTOR (should be the fastest, haven't tried in this branch):
-    //BEWARE: failing in one noodlificatoin test ("Simple automata -- epsilon result") ... strange
-    //BoolVector chv = get_used_symbols_chv();
-    //utils::OrdVector<Symbol> ov;
-    //for(Symbol i = 0;i<chv.size();i++)
-    //    if (chv[i]) {
-    //        ov.push_back(i);
-    //    }
-    //return ov;
+Nfa& Nfa::complement_deterministic(const OrdVector<Symbol>& symbols, std::optional<State> sink_state) {
+    const State sink{ sink_state.value_or(num_of_states()) };
+    if (initial.empty()) { // The automaton has no reachable states (accepting an empty language).
+        // Insert a single initial sink state.
+        initial.insert(sink);
+    }
+    make_complete(symbols, sink);
+    swap_final_nonfinal();
+    return *this;
+}
+
+Nfa& Nfa::unite_nondet_with(const mata::nfa::Nfa& aut) {
+    const size_t num_of_states{ this->num_of_states() };
+    const size_t aut_num_of_states{ aut.num_of_states() };
+    const size_t new_num_of_states{ num_of_states + aut_num_of_states };
+
+    if (this == &aut) {
+        return *this;
+    }
+
+    if (final.empty() || initial.empty()) { *this = aut; return *this; }
+    if (aut.final.empty() || aut.initial.empty()) { return *this; }
+
+    this->delta.reserve(new_num_of_states);
+    // Allocate space for initial and final states from 'this' which might be missing in Delta.
+    this->delta.allocate(num_of_states);
+
+    auto renumber_states = [&](State st) {
+        return st + num_of_states;
+    };
+    this->delta.append(aut.delta.renumber_targets(renumber_states));
+
+    // Set accepting states.
+    this->final.reserve(new_num_of_states);
+    for(const State& aut_fin: aut.final) {
+        this->final.insert(renumber_states(aut_fin));
+    }
+    // Set initial states.
+    this->initial.reserve(new_num_of_states);
+    for(const State& aut_ini: aut.initial) {
+        this->initial.insert(renumber_states(aut_ini));
+    }
+
+    return *this;
 }
