@@ -1,18 +1,4 @@
 /* parser.cc -- implementation of VTF format parser
- *
- * Copyright (c) 2018 Ondrej Lengal <ondra.lengal@gmail.com>
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include "mata/parser/parser.hh"
@@ -199,14 +185,14 @@ std::vector<std::pair<std::string, bool>> tokenize_line(const std::string& line)
 
 std::vector<std::pair<std::string, bool>> split_tokens(std::vector<std::pair<std::string, bool>> tokens) {
     std::vector<std::pair<std::string, bool>> result;
-    for (const auto& token : tokens) {
+    for (auto& token: tokens) {
         if (token.second) { // is quoted?
             result.push_back(std::move(token));
             continue;
         }
 
-        const std::string_view token_string = token.first;
-        size_t last_operator = 0;
+        const std::string_view token_string{ token.first };
+        size_t last_operator{ 0 };
         for (size_t i = 0, token_string_size{ token_string.size() }; i < token_string_size; ++i) {
             if (is_logical_operator(token_string[i])) {
                 const std::string_view token_candidate = token_string.substr(last_operator, i - last_operator);
