@@ -1,18 +1,6 @@
 /* re2parser.cc -- parser transforming re2 regular expressions to our Nfa
  *
  * Copyright (c) 2022 Michal Horky
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include <iostream>
@@ -488,8 +476,8 @@ namespace {
                         if (renumbered_states[stateTo] == mata::nfa::Limits::max_state) {
                             renumbered_states[stateTo] = renumbered_explicit_nfa.add_state();
                         }
-                        assert(renumbered_states[state] <= renumbered_explicit_nfa.size());
-                        assert(renumbered_states[stateTo] <= renumbered_explicit_nfa.size());
+                        assert(renumbered_states[state] <= renumbered_explicit_nfa.num_of_states());
+                        assert(renumbered_states[stateTo] <= renumbered_explicit_nfa.num_of_states());
                         renumbered_explicit_nfa.delta.add(renumbered_states[state], transition.symbol,
                                                           renumbered_states[stateTo]);
                     }
