@@ -1,18 +1,4 @@
 /* nfa-segmentation.cc -- Segmentation of NFAs
- *
- * Copyright (c) 2022 David Chocholatý <chocholaty.david@protonmail.com>
- *
- * This file is a part of libmata.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
 #include "mata/nfa/strings.hh"
@@ -45,13 +31,14 @@ void seg_nfa::Segmentation::handle_epsilon_transitions(const StateDepthTuple& st
 
     for (State target_state: move.targets)
     {
-        this->epsilon_depth_transitions[state_depth_pair.depth].push_back(
-                Transition{ state_depth_pair.state, move.symbol, target_state }
+        // TODO: Use vector indexed by depths instead of a map.
+        this->epsilon_depth_transitions[state_depth_pair.depth].emplace_back(
+                state_depth_pair.state, move.symbol, target_state
         );
-        this->eps_depth_trans_map[state_depth_pair.depth][state_depth_pair.state].push_back(
-                Transition{ state_depth_pair.state, move.symbol, target_state }
+        this->eps_depth_trans_map[state_depth_pair.depth][state_depth_pair.state].emplace_back(
+             state_depth_pair.state, move.symbol, target_state
         );
-        worklist.push_back(StateDepthTuple{ target_state, state_depth_pair.depth + 1, visited_eps_aux });
+        worklist.push_back({ target_state, state_depth_pair.depth + 1, visited_eps_aux });
         this->visited_eps[target_state] = visited_eps_aux;
     }
 }
@@ -85,7 +72,7 @@ std::deque<seg_nfa::Segmentation::StateDepthTuple> seg_nfa::Segmentation::initia
 std::unordered_map<State, bool> seg_nfa::Segmentation::initialize_visited_map() const
 {
     std::unordered_map<State, bool> visited{};
-    const size_t state_num = automaton.size();
+    const size_t state_num = automaton.num_of_states();
     for (State state{ 0 }; state < state_num; ++state)
     {
         visited[state] = false;
